"test project service"
import os
import sys
from unittest.mock import patch
import pytest
from gamma.cli.services.project import ProjectService
from mockups.github_organisation import GithubOrganization
from mockups.github_user import GithubUser
from mockups.github_service import github, MOCK_TOKEN

# pylint:disable=protected-access

PACKAGE_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, PACKAGE_PATH + "/..")


@pytest.fixture(name="project_svc")
def build_project_svc():
    "fixture returning a project service plugged to a mock github repo"
    return ProjectService(gitHubService=github)


@patch("gamma.cli.services.project.cookiecutter")
def test_create_cookiecutter(mock_cookiecutter, project_svc):
    "call execcookie cutter, and makes sure it calls correctly the cookiecutter lib"
    path = "random_path"
    project_svc._exec_cookiecutter(path)  # pylint: disable=protected-access
    mock_cookiecutter.assert_called_with(
        "https://{}@github.gamma.bcg.com/{}".format(MOCK_TOKEN, project_svc.base_repo),
        output_dir="random_path",
    )


NO_GITHUB_CMD = "n\n"
@patch("gamma.cli.services.project.Repo")
@patch("gamma.cli.services.project.cookiecutter")
@patch("builtins.input", side_effect=[NO_GITHUB_CMD])
def test_new_without_remote(_inp, mock_cookiecutter, mock_gitrepo, project_svc):
    "makes sure that, when called without asking for a remote, cookie cutter is called, then git, but no ref to github"

    # set up: we have a fake repo, and a user saying no to remote repo
    fake_repo_path = "/fake/path/file"
    # patching things up
    mock_cookiecutter.return_value = fake_repo_path # I dont actually want to create a repo
    def fail_ifcalled():
        raise AssertionError("github should not be called")
    project_svc.github.side_effect = fail_ifcalled # fail the test if anything from github is called

    project_svc.new(fake_repo_path)
    mock_gitrepo.init.assert_called_with(fake_repo_path)


YES_GITHUB_CMD = ["y\n", "1\n", "y\n"]
@patch("gamma.cli.services.project.Repo")
@patch("gamma.cli.services.project.cookiecutter")
@patch("builtins.input", side_effect=YES_GITHUB_CMD)
def test_new_with_remote(_inp, mock_cookiecutter, mock_gitrepo, project_svc):
    "emulates calling a new project from github, and makes sure it is called regularly"

    # set up: we have a fake repo, and a user saying no to remote repo
    repo_name = "repo_name"
    fake_repo_path = "/fake/path/" + repo_name
    # patching things up
    mock_cookiecutter.return_value = fake_repo_path # I dont actually want to create a repo

    project_svc.new(fake_repo_path)
    mock_gitrepo.init.assert_called_with(fake_repo_path)
    project_svc.github_user.create_repo.assert_called_with(repo_name, private=False)

def test_fix_path():
    "test fix path"
    project = ProjectService()

    assert project._fix_new_path(None) == "."
    assert project._fix_new_path("/tmp") == "/tmp"


def test_selector_org_options():
    "test selector"
    project = ProjectService()
    project.github_user = GithubUser()
    project.github_user.login = "username"

    lst = []
    org = GithubOrganization()
    org.login = "test"
    org.description = "desc"
    lst.append(org)
    org = GithubOrganization()
    org.login = "test2"
    lst.append(org)

    data = project._get_orgs_selector_option(lst)
    assert len(data) == 4
    assert data[0] == "test - desc"
    assert data[1] == "test2"
    assert data[2] == "Create a new organization for this project/case"
    assert data[3] == "username - Your personal account"
