import click
import pytest


@pytest.fixture()
def setup_plugin(monkeypatch):
    monkeypatch.setenv("GAMMA_PLUGINS_LOGLEVEL", "debug")

    from gamma.cli import plugins

    @click.group()
    def sample():
        """Group to be attached to the root"""

    @sample.command()
    def sample_sub():
        """This will be attached automatically, no need to pass."""

    @click.command()
    def foo():
        """ This will be renamed to bar """

    @click.command()
    def subnew():
        """ This will be attached to the existing 'new' group"""

    class PluginImpl:
        """ Implement plugin into a class to simplify registration """

        @plugins.hookimpl
        def add_commands(self):
            from gamma.cli.plugins import Command

            return [
                Command(sample),
                Command(foo, name="bar"),
                Command(subnew, parent="new"),
            ]

    impl = PluginImpl()
    plugins.plugin_manager.register(impl, name="plugins_test.PluginImpl")
    yield
    plugins.plugin_manager.unregister(impl)


def test_plugins(setup_plugin):
    from gamma.cli.gamma import CLI

    # initialize the root group
    cli = CLI("root")

    assert "sample" in cli.commands
    assert "sample-sub" in cli.commands["sample"].commands
    assert "bar" in cli.commands
    assert "subnew" in cli.commands["new"].commands
