"Behave feature steps for login"
import logging
import os
import subprocess
import sys

import pexpect as P

from behave import given, then, when


TOKEN_PATH = "./token"
logger = logging.getLogger(__name__)

# The token we use for checking the account is a very special token, with very limited rights. it is associated with anselme vignon acount,
#    and will fail when this account is disabled.
#    to create a 'valid' replacement token,
#      - got to: https://github.gamma.bcg.com/settings/tokens
#      - create a token, with ONLY the scope "read-user" and "read:public_key" activated (anything else would be a security concern).
#      - copy paste the generated token into the "test_token" file
with open("tests/test_token", "r") as f:
    LIMITED_ACCESS_TOKEN = f.read()


@given('gamma tool')
def g_tool(_context):
    "test gamma tool: display help"
    os.environ["GITHUB_TOKEN_PATH"] = TOKEN_PATH
    print(subprocess.Popen("python -m gamma.cli.gamma", shell=True, stdout=subprocess.PIPE).stdout.read())


@when(u"we run gamma login")
def g_login(_context):
    "send the login command"
    child = P.spawn("python -m gamma.cli.gamma login", encoding="utf-8")
    child.logfile = sys.stdout
    child.expect("Copy the personal access token and paste it here:")
    child.sendline(LIMITED_ACCESS_TOKEN)
    child.expect(P.EOF)
    child.wait()
    assert child.exitstatus == 0


@then(u"the token file is created")
def token_exists(_context):
    "check that the token is generated"
    assert os.path.exists(TOKEN_PATH), "token file created"


@then(u"we can login without input")
def g_login_notoken(_context):
    "uses the written login to log in"
    child = P.spawn("python -m gamma.cli.gamma login", encoding="utf-8")
    child.logfile = sys.stdout
    child.expect(P.EOF)
    child.wait()
    assert child.exitstatus == 0
