"""
Plugin used to connect the tool to Github.

Will offer the user to open the url: https://github.gamma.bcg.com/settings/tokens
to create a new token.

The toolbox will check if the user has a valid ssh key and ask the
user to generate one if missing

The users must give all permissions except:

- admin:gpg_key

"""

import click

from gamma.cli.services.github import GitHubService, delete_token_file, validate_token


@click.command()
@click.pass_obj
def cli(telemetry):
    """ Allows connection with Github """

    if telemetry is not None:
        telemetry.track_event("login cmd")

    git = GitHubService()

    try:
        validate_token(git.token, svc=git)

        if telemetry is not None:
            telemetry.track_event("github login OK")
            telemetry.flush()
    except:
        delete_token_file()
        print("Invalid or expired token, please try the login command again")
        if telemetry is not None:
            telemetry.track_event("github login not OK")
            telemetry.flush()
